#include "AvoidanceBehaviour.h"
#include <visualization_msgs/MarkerArray.h>

using namespace std;

AvoidanceBehaviour::AvoidanceBehaviour(Transformhandler* ReferenceTransformhandler,
                                       TaskLevelController* OwningTaskLevelController,
                                       uint OperationPoint,
                                       double range,
                                       ros::NodeHandle* n,
                                       ObstaclesHandler* obstacles
                                )
  : Behaviour(ReferenceTransformhandler,OwningTaskLevelController), myObstaclesHandler(obstacles), myRange(range)
{
  myTorques.setZero();

  spheres_pub = n->advertise<visualization_msgs::MarkerArray>("visualization_marker_array", 1);

  OperationPointIndex = OperationPoint;

  OperationalDOFMask[MOVE_X] = true;
  OperationalDOFMask[MOVE_Y] = true;
  OperationalDOFMask[MOVE_Z] = true;
  OperationalDOFMask[ROTATE_X] = false;
  OperationalDOFMask[ROTATE_Y] = false;
  OperationalDOFMask[ROTATE_Z] = false;

  BehaviorActive = false;
  TaskReferenceAcceleration.setZero(3,1);
  //***Defining reference points of the robot and their respective ranges**//
  if (OperationPoint == 3){
    //points in the links 3 and 4 (viewed from the frame 3)
    pointsSegment_1.push_back(btVector3(0,0,0.218));
    pointRanges.push_back(0.056);
    pointsSegment_1.push_back(btVector3(0,0.026,0.218));
    pointRanges.push_back(0.056);
    pointsSegment_1.push_back(btVector3(0,-0.026,0.218));
    pointRanges.push_back(0.056);
    pointsSegment_1.push_back(btVector3(0,0,0.162));
    pointRanges.push_back(0.056);
    pointsSegment_1.push_back(btVector3(0,0,0.106));
    pointRanges.push_back(0.056);
    pointsSegment_1.push_back(btVector3(0,0,0.054));
    pointRanges.push_back(0.056);
//    pointsSegment_1.push_back(btVector3(0.,0,-0.002));
//    pointRanges.push_back(0.056);
  }
  else
    if(OperationPoint == 5){
      //points in the links 5 and 6 (viewed from the frame 5)
      pointsSegment_2.push_back(btVector3(0,0,0.177));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0,0.026,0.177));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0,-0.026,0.177));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0,0,0.132));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0,0,0.087));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0,0,0.042));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0.,0,-0.003));
      pointRanges.push_back(0.045);
      pointsSegment_2.push_back(btVector3(0.,0,-0.048));
      pointRanges.push_back(0.045);
    }
    else
      if (OperationPoint == 6){
        //points in the link 7 (viewed from the frame 7)
        pointsSegment_3.push_back(btVector3(0,0,-0.038));
        pointRanges.push_back(0.038);
        pointsSegment_3.push_back(btVector3(0.,0,-0.075));
        pointRanges.push_back(0.038);
        pointsSegment_3.push_back(btVector3(0.,0,-0.114));
        pointRanges.push_back(0.038);
      }
      else
        cout<<"AvoidanceBehaviour::calculateReferenceAcceleration: Wrong OperationPointIndex!"<<endl;

  mainpointIsCloseToCellTreshold[0] = 0.3 + CELL_GRID_STEP*2;
  mainpointIsCloseToCellTreshold[1] = 0.2 + CELL_GRID_STEP*2;
  mainpointIsCloseToCellTreshold[2] = 0.1 + CELL_GRID_STEP*2;
}

AvoidanceBehaviour::~AvoidanceBehaviour()
{}

void AvoidanceBehaviour::setRange(double range)
{
  myRange = range;
}

void AvoidanceBehaviour::setObstaclesHandler(ObstaclesHandler* obstacles)
{
  myObstaclesHandler = obstacles;
}


void AvoidanceBehaviour::calculateReferenceAcceleration()
{
  TaskReferenceAcceleration.setZero(3,1);
  BehaviorActive = false;
  TaskJacobian.setZero();
  myTorques.setZero();
  if(myObstaclesHandler==NULL)
    return;
  vector<Obstacle*> obstacles = myObstaclesHandler->getObstacles();
  if (obstacles.size() == 0)   // No obstacles to be repelled from
    return;

  vector<btTransform> transforms = BehaviorTransformhandler->getRootToJointTransforms();


  vector<btVector3> pointsTransformed;
  switch (OperationPointIndex){
    case (3):
      for (uint i=0; i<pointsSegment_1.size(); i++)
        pointsTransformed.push_back(transforms[2] * pointsSegment_1[i]);
      break;
    case (5):
      for (uint i=0; i<pointsSegment_2.size(); i++)
        pointsTransformed.push_back(transforms[4] * pointsSegment_2[i]);
      break;
    case (6):
      for (uint i=0; i<pointsSegment_3.size(); i++)
        pointsTransformed.push_back(transforms[6] * pointsSegment_3[i]);
      break;
    default:
      cout<<"AvoidanceBehaviour::calculateReferenceAcceleration: Wrong OperationPointIndex!"<<endl;
      TaskReferenceAcceleration.setZero();
      TaskJacobian.setZero();
      BehaviorActive = false;
      return;
  }




//  ///****publishing spheres  ****///
//
//  visualization_msgs::MarkerArray spheresMarkers;
//  spheresMarkers.markers.resize(pointsTransformed.size());
//
//  vector<btVector3>::iterator it=pointsTransformed.begin();
//
//  btQuaternion tmp_trans;
//  for (uint i=0; i<pointsTransformed.size(); i++)
//  {
//    char ns[50];
//    sprintf (ns, "ObstacleAvoidanceSpheres%d", OperationPointIndex);
//    spheresMarkers.markers[i].ns = ns ;
//    spheresMarkers.markers[i].header.stamp = ros::Time::now();
//    spheresMarkers.markers[i].header.frame_id = "/fix";
//    spheresMarkers.markers[i].id = i;
//    spheresMarkers.markers[i].type =visualization_msgs::Marker::SPHERE;
//      spheresMarkers.markers[i].action = visualization_msgs::Marker::ADD;
//    spheresMarkers.markers[i].pose.position.x = (*it).getX();
//    spheresMarkers.markers[i].pose.position.y = (*it).getY();
//    spheresMarkers.markers[i].pose.position.z = (*it).getZ();
//    tmp_trans = btQuaternion(btVector3(0,0,1),0);
//    spheresMarkers.markers[i].pose.orientation.x = tmp_trans.getX();
//    spheresMarkers.markers[i].pose.orientation.y = tmp_trans.getY();
//    spheresMarkers.markers[i].pose.orientation.z = tmp_trans.getZ();
//    spheresMarkers.markers[i].pose.orientation.w = tmp_trans.getW();
//
//    spheresMarkers.markers[i].scale.x = pointRanges[i]*2;
//    spheresMarkers.markers[i].scale.y = pointRanges[i]*2;
//    spheresMarkers.markers[i].scale.z = pointRanges[i]*2;
//
//    spheresMarkers.markers[i].lifetime = ros::Duration();
//    if (OperationPointIndex == 5)
//    {
//      spheresMarkers.markers[i].color.r = 1.0f;
//      spheresMarkers.markers[i].color.g = 1.0f;
//      spheresMarkers.markers[i].color.b = 0.0f;
//      spheresMarkers.markers[i].color.a = 1.0;
//    }
//    else
//      if (OperationPointIndex == 6)
//      {
//        spheresMarkers.markers[i].color.r = 0.0f;
//        spheresMarkers.markers[i].color.g = 1.0f;
//        spheresMarkers.markers[i].color.b = 1.0f;
//        spheresMarkers.markers[i].color.a = 1.0;
//      }
//      else
//      {
//        spheresMarkers.markers[i].color.r = 0.5f;
//        spheresMarkers.markers[i].color.g = 0.5f;
//        spheresMarkers.markers[i].color.b = 0.5f;
//        spheresMarkers.markers[i].color.a = 1.0;
//      }
//    it++;
//  }
//  if(ros::ok())
//    spheres_pub.publish(spheresMarkers);
//  /*---------------END MARKER------------*/



  btVector3 mainPoints[3];
  mainPoints[0] = transforms[2].getOrigin();
  mainPoints[1] = transforms[4].getOrigin();
  mainPoints[2] = transforms[6].getOrigin();

  vector<btVector3> pointforces;
  pointforces.resize(pointsTransformed.size());
  for (uint i=0; i<pointforces.size(); i++)
  {
    //cout<<"point "<<i<<": "<<pointsTransformed[i][0]<<" "<<pointsTransformed[i][1]<<" "<<pointsTransformed[i][2]<<endl;
    pointforces[i] = btVector3(0,0,0);
  }


  map<long int, Cell*>::iterator cell_it;
  //loop through all obstacles
  for (uint obst_ctr=0; obst_ctr<obstacles.size(); obst_ctr++){
    //loop through all cells in obstacles
    for(cell_it = obstacles[obst_ctr]->occupied_cells.begin(); cell_it != obstacles[obst_ctr]->occupied_cells.end();cell_it++){
      //check, if this cell is far away from any of the main points of robot. if so, go to next cell
      btVector3 cellCenter = (*cell_it).second->getCenter();

      if ((mainPoints[0] - cellCenter).length() >  mainpointIsCloseToCellTreshold[0] &&
          (mainPoints[1] - cellCenter).length() >  mainpointIsCloseToCellTreshold[1] &&
          (mainPoints[2] - cellCenter).length() >  mainpointIsCloseToCellTreshold[2])
        continue;

      //loop through all points
      for (uint i=0; i<pointsTransformed.size(); i++){
//        btVector3 b = pointsTransformed[i];
        btVector3 c = pointsTransformed[i] - cellCenter;
        if (c.length() < pointRanges[i] + myRange +CELL_GRID_STEP/2 )
        {
          //force module grows when the distance between cell and point diminishes
//          double d1 = c.length(); double d2 = pointRanges[i]; double d3 = CELL_GRID_STEP*sqrt(3);
//          double d = 1/(c.length() - pointRanges[i] );
          double force = 1/(c.length() - pointRanges[i] - CELL_GRID_STEP/2) - 1/myRange;
          if (c.length() - pointRanges[i] - CELL_GRID_STEP/2 < 0.1*myRange)
            force = 9/myRange;
          pointforces[i] = pointforces[i] + c.normalized()* force;
        }

      }//going through all points
    }//going through all cells of an obstacle
  }//going through all obstacles

  //summing up of all forces inside of segments
  btVector3 totalForceSegment;
  for (uint i=0; i<pointforces.size(); i++)
    totalForceSegment = totalForceSegment + pointforces[i];


  Eigen::Vector3d DesiredSpeedVector;

  double Kp = 1, Kv = 10;
  double Point6maxSpeed = 0.08;
  double Point5maxSpeed = 0.07;
  double Point3maxSpeed = 0.06;

  if (totalForceSegment.length() > 0)
  {
    if (OperationPointIndex == 6)
    {
      if (totalForceSegment.length()> Point6maxSpeed)
        totalForceSegment = totalForceSegment.normalized()*Point6maxSpeed;
    }
    else
      if (OperationPointIndex == 5)
      {
        if (totalForceSegment.length()> Point5maxSpeed)
          totalForceSegment = totalForceSegment.normalized()*Point5maxSpeed;
      }
      else
        if (OperationPointIndex == 3)
        {
          if (totalForceSegment.length()> Point3maxSpeed)
            totalForceSegment = totalForceSegment.normalized()*Point3maxSpeed;
        }

    TaskJacobian = calculateOperationPointJacobian(OperationPointIndex);
    DesiredSpeedVector[0] = totalForceSegment[0];
    DesiredSpeedVector[1] = totalForceSegment[1];
    DesiredSpeedVector[2] = totalForceSegment[2];
    BehaviorActive=true;
  }
  else
  {
    BehaviorActive = false;
    return;
  }


  //cout<<"Oper.point "<<OperationPointIndex<<"; DesiredSpeedVector= "<<DesiredSpeedVector[0]<<" "<<DesiredSpeedVector[1]<<" "<<DesiredSpeedVector[2];

  DesiredSpeedVector = Kp * DesiredSpeedVector;
//  cout<<TaskJacobian<<endl;
//  cout<<"speed vector: "<<myTaskLevelController->getSpeedVector().transpose()<<endl;
  Eigen::Vector3d ActualSpeedVector = TaskJacobian * myTaskLevelController->getSpeedVector();
//  cout<<" ActualSpeedVector ="<<ActualSpeedVector.transpose()<<endl;
  TaskReferenceAcceleration = -Kv * (ActualSpeedVector - DesiredSpeedVector);
}


