#ifndef __AVOIDANCEBEHAVIOUR_H__
#define __AVOIDANCEBEHAVIOUR_H__

#include "Behaviour.h"
#include "../ObstaclesHandler.h"
#include <vector>
#include <ros/ros.h>
/**
   @brief Maximum force vector length
*/
#define AVOIDANCEBEHAVIOUR_MAXSPEED_ARMPOINT 0.5

class ObstaclesHandler;

/**@brief the behavior to avoid obstacles.
 * Currently it is only checked if the origins of the link-related
 * coordinate frames are inside the bounding box (TODO: check the whole rigid bodies).
 * The origins are checked one by one from the base to the end effector; the first origin colliding
 * with an obstacle is set as an operating point for a repulsive force.
 * If no collisions detected, the behavior is inactive.
 */



class AvoidanceBehaviour : public Behaviour {
 public:

	AvoidanceBehaviour(Transformhandler* ReferenceTransformhandler,
			 TaskLevelController* OwningTaskLevelController,
			 uint OperationPoint,
			 double range = 0.05,
			 ros::NodeHandle* n = NULL,
			 ObstaclesHandler* obstacles = NULL
		  );
  virtual ~AvoidanceBehaviour();
  void setRange(double range);
  void setObstaclesHandler(ObstaclesHandler* obstacles);

 protected:
  void calculateReferenceAcceleration();

  ObstaclesHandler* myObstaclesHandler;
  /** @brief Max range for an obstacle to influence the robot */
  double myRange;
  vector<btVector3> pointsSegment_1;
  vector<btVector3> pointsSegment_2;
  vector<btVector3> pointsSegment_3;

  vector<double> pointRanges;

  ros::Publisher spheres_pub;

  double mainpointIsCloseToCellTreshold[3];
};

#endif /*__AVOIDANCEBEHAVIOUR_H__*/
