/*
 * EndEffectorBehavior.cpp
 *
 *  Created on: 05.05.2012
 *      Author: sergey
 */

#include "EndEffectorBehavior.h"

EndEffectorBehavior::EndEffectorBehavior(Transformhandler* ReferenceTransformhandler,
                                         TaskLevelController* OwningTaskLevelController)
                                        : Behaviour(ReferenceTransformhandler, OwningTaskLevelController)
{
  BehaviorActive = false;

  OperationPointIndex = BehaviorTransformhandler->getOrigins().size()-1;

  TaskReferenceAcceleration.setZero(3,1);

  OperationalDOFMask[MOVE_X] = false;
  OperationalDOFMask[MOVE_Y] = false;
  OperationalDOFMask[MOVE_Z] = false;
  OperationalDOFMask[ROTATE_X] = true;
  OperationalDOFMask[ROTATE_Y] = true;
  OperationalDOFMask[ROTATE_Z] = true;

  TaskType = EndEffectorBehavior::NO_TASK;
  desiredFixedOrientation.setZero();
  pointToAim.setZero();
}

EndEffectorBehavior::~EndEffectorBehavior()
{
  // TODO Auto-generated destructor stub
}

/* **
   @brief Specifies a goal of the behavior as pointing with end effector at a point
*/
void EndEffectorBehavior::setGoalPoint(const btVector3& goalPoint)
{
        pointToAim = goalPoint;
        std::cout<<"New End Effector Task: aim at the point ("<<goalPoint.getX()<<" "<<goalPoint.getY()<<" "<<goalPoint.getZ()<<") "<<std::endl;
        TaskType = EndEffectorBehavior::GOAL_AIM_AT_POINT;
        BehaviorActive = true;
}

void EndEffectorBehavior::setGoalOrientation(const btVector3& orient)
{
  desiredFixedOrientation = orient.normalized();
  TaskType = EndEffectorBehavior::GOAL_PRESERVE_ORIENTATION;
  std::cout<<"New End Effector Task: preserve fixed orientation ("<<orient.getX()<<" "<<orient.getY()<<" "<<orient.getZ()<<") "<<std::endl;
  BehaviorActive = true;
}

btVector3 EndEffectorBehavior::getGoal(uint& goalType)
{
  goalType = TaskType;
  switch (TaskType){
    case (EndEffectorBehavior::GOAL_PRESERVE_ORIENTATION):
        return desiredFixedOrientation;
    case (EndEffectorBehavior::GOAL_AIM_AT_POINT):
        return pointToAim;
    default:
      return btVector3(0,0,0);
  }
}

const torqueVector& EndEffectorBehavior::getTorque()
{
  calculateReferenceAcceleration();
  calculateNextTaskNullspace();
  return myTorques;
}


//--TODO fix here
void EndEffectorBehavior::calculateReferenceAcceleration()
{
  btVector3 desiredEEaxis,currentEEPos;
  switch (TaskType){
    case (EndEffectorBehavior::GOAL_AIM_AT_POINT):
      currentEEPos = BehaviorTransformhandler->getOrigins().at(NumberOfDOF-1);
      desiredEEaxis = -(pointToAim - currentEEPos).normalize();
      break;
    case (EndEffectorBehavior::GOAL_PRESERVE_ORIENTATION):
      desiredEEaxis = -desiredFixedOrientation;
      break;
    default:
      BehaviorActive = false;
      break;
  }

  if (!BehaviorActive)
  {
          TaskReferenceAcceleration.setZero();
          TaskJacobian.setZero();
          return;
  }

  Eigen::MatrixXd TaskJacobianBuf = calculateOperationPointJacobian(OperationPointIndex);
  TaskJacobian.setZero(TaskJacobianBuf.rows(),TaskJacobianBuf.cols());

  //Getting data about configuration
  cVector q_current = BehaviorTransformhandler->getConfiguration();

  torqueVector Desired_q_speed;
  Desired_q_speed.setZero();
  torqueVector Actual_q_speed = myTaskLevelController->getSpeedVector();
  torqueVector Desired_q_acceleration;
  Desired_q_acceleration.setZero();

  double Kp = 10;
  double Kv = 15;

  btQuaternion ZtoY, YtoZ;
  ZtoY = btQuaternion(btVector3(1,0,0), -SIMD_PI*.5) * btQuaternion(btVector3(0,0,1), -SIMD_PI*.5);
  YtoZ = ZtoY.inverse();
btTransform tmp;
tmp.setRotation(YtoZ);

  btVector3 VectorFromFrame4 = -((BehaviorTransformhandler->getRootToJointTransform(3) * tmp).getBasis().inverse()*desiredEEaxis);
  btMatrix3x3 mat = BehaviorTransformhandler->getRootToJointTransform(6).getBasis();
  double x=VectorFromFrame4[0],y=VectorFromFrame4[1],z=VectorFromFrame4[2];
  VectorFromFrame4 = BehaviorTransformhandler->getRootToJointTransform(4).getBasis()*btVector3(0,0,1);

  double theta, phi;
  phi = acos(z);
  if (x==0 and y==0)
    theta = 0;
  else
    theta = -atan2(y,x);

  double delta4 = theta - q_current[4];
  double delta5 = phi - q_current[5];
  if (delta4 >= M_PI)
    delta4 -= 2*M_PI;
  if (delta4 <= -M_PI)
    delta4 += 2*M_PI;

  if (delta5 >= M_PI)
    delta5 -= 2*M_PI;
  if (delta5 <= -M_PI)
    delta5 += 2*M_PI;

 // std::cout<<"theta(4)="<<theta<<", phi(5)="<<phi<<std::endl;

  Desired_q_speed[4] = Kp * (delta4);
//  if (fabs(Desired_q_speed[4]) > 0.44)
//    Desired_q_speed[4] = 0.44 * signof(Desired_q_speed[4]);
  Desired_q_acceleration[4] = -Kv * (Actual_q_speed[4] - Desired_q_speed[4]);
//  if (fabs(Desired_q_acceleration[4]) > 1.75)
//    Desired_q_acceleration[4] = 1.75 * signof(Desired_q_speed[4]);
  TaskJacobian.col(4) = TaskJacobianBuf.col(4);

  Desired_q_speed[5] = Kp * (delta5);
//  if (fabs(Desired_q_speed[5]) > 0.55)
//    Desired_q_speed[5] = 0.55 * signof(Desired_q_speed[5]);
  Desired_q_acceleration[5] = -Kv * (Actual_q_speed[5] - Desired_q_speed[5]);
//  if (fabs(Desired_q_acceleration[5]) > 1.75)
//    Desired_q_acceleration[5] = 1.75 * signof(Desired_q_speed[5]);
  TaskJacobian.col(5) = TaskJacobianBuf.col(5);

  //TODO: Compute Coriolis and gravitation parts with Newton-Euler algorithm and
  //                    add their contribution to the torques
  torqueVector DesiredTorques = myTaskLevelController->getInertiaMatrix() * Desired_q_acceleration;

  Eigen::MatrixXd N_prev = myTaskLevelController->getNullSpaceMatrix();
  myTorques = N_prev.transpose() * DesiredTorques;

  BehaviorActive = DesiredTorques.norm() > 0;

}



//void EndEffectorBehavior::calculateReferenceAcceleration()
//{
//  if (!BehaviorActive)
//  {
//          TaskReferenceAcceleration.setZero();
//          TaskJacobian.setZero();
//          return;
//  }
//
//  btVector3 EEaxis = BehaviorTransformhandler->getAxes().at(NumberOfDOF-1);
//  btVector3 currentEEPos = BehaviorTransformhandler->getOrigins().at(NumberOfDOF-1);
//  btVector3 desiredEEaxis = -(pointToAim - currentEEPos).normalize();
//
//  //alpha, beta, gamma - angles around X, Y, Z axes
//  double actualAlpha, actualBeta, actualGamma;
//  double desiredAlpha, desiredBeta, desiredGamma;
//
//  actualAlpha = atan2(EEaxis.getZ(), EEaxis.getY());
//  actualBeta = atan2(EEaxis.getZ(), -EEaxis.getX());
//  actualGamma = atan2(EEaxis.getY(), EEaxis.getX());
//
//  desiredAlpha = atan2(desiredEEaxis.getZ(), desiredEEaxis.getY());
//  desiredBeta = atan2(desiredEEaxis.getZ(), -desiredEEaxis.getX());
//  desiredGamma = atan2(desiredEEaxis.getY(), desiredEEaxis.getX());
//
//  double Kp = 0.25;
//  double Kv = 0.2;
//
//  Eigen::Vector3d desiredSpeedVector;
//  desiredSpeedVector <<desiredAlpha - actualAlpha, desiredBeta - actualBeta, desiredGamma - actualGamma;
//
//  desiredSpeedVector = desiredSpeedVector * Kp;
//  TaskJacobian = calculateOperationPointJacobian(NumberOfDOF-1);
//
//
//  Eigen::Vector3d ActualSpeedVector = TaskJacobian * myTaskLevelController->getSpeedVector();
//
//  TaskJacobian.leftCols(4).setZero();
//
//  std::cout<<std::endl<<"TaskJacobian:"<<std::endl<<TaskJacobian<<std::endl;
//  std::cout<<"current axis: "<<EEaxis[0]<<" "<<EEaxis[1]<<" "<<EEaxis[2]<<"; desired axis: "<<desiredEEaxis[0]<<" "<<desiredEEaxis[1]<<" "<<desiredEEaxis[2]<<std::endl;
//  std::cout<<"actual angles: alpha="<<actualAlpha<<" beta="<<actualBeta<<" gamma="<<actualGamma<<"; des angles: alpha="<<desiredAlpha<<" beta="<<desiredBeta<<" gamma="<<desiredGamma<<std::endl;
//  TaskReferenceAcceleration = -Kv * (ActualSpeedVector - desiredSpeedVector);
//
//}
