/*
 * GlobalBehaviour.cpp
 *
 *  Created on: 18.11.2011
 *      Author: sergey
 */

#include "GlobalBehaviour.h"

GlobalBehaviour::GlobalBehaviour(Transformhandler* ReferenceTransformhandler,
		 TaskLevelController* OwningTaskLevelController)
		: Behaviour(ReferenceTransformhandler, OwningTaskLevelController)
{
  BehaviorActive = false;

  OperationPointIndex = BehaviorTransformhandler->getOrigins().size()-1;

  TaskReferenceAcceleration.setZero(3,1);

  OperationalDOFMask[MOVE_X] = true;
  OperationalDOFMask[MOVE_Y] = true;
  OperationalDOFMask[MOVE_Z] = true;
  OperationalDOFMask[ROTATE_X] = false;
  OperationalDOFMask[ROTATE_Y] = false;
  OperationalDOFMask[ROTATE_Z] = false;

  myGoalPoint = btVector3(0,0,0);
}

GlobalBehaviour::~GlobalBehaviour() {}

/*@brief Makes the behavior inactive*/
void GlobalBehaviour::disableBehaviour()
{
  BehaviorActive = false;
  myGoalPoint.setZero();
}

/* **
   @brief Specifies a goal of the behavior as keeping close to a point
*/
void GlobalBehaviour::setGoalPoint(const btVector3& goalPoint)
{
        myGoalPoint = goalPoint;
        std::cout<<"newGoal: "<<goalPoint.getX()<<" "<<goalPoint.getY()<<" "<<goalPoint.getZ()<<" "<<std::endl;
        btVector3 endEffectorPos = BehaviorTransformhandler->getOrigins()[6];
        std::cout<<"Current pos: "<<endEffectorPos.getX()<<" "<<endEffectorPos.getY()<<" "<<endEffectorPos.getZ()<<" "<<std::endl;

        BehaviorActive = true;
}

btVector3 GlobalBehaviour::getGoalPoint()
{
  return myGoalPoint;
}

void GlobalBehaviour::calculateReferenceAcceleration()
{
    Eigen::Vector3d DesiredSpeedVector;
    DesiredSpeedVector.setZero();
    if (!BehaviorActive)
    {
            TaskReferenceAcceleration.setZero();
            TaskJacobian.setZero();
            return;
    }
    std::vector<tf::Vector3> currentOrigins;
    currentOrigins = BehaviorTransformhandler->getOrigins();
    btVector3 endEffectorPos = currentOrigins[currentOrigins.size()-1];
    btVector3 Point2PointVec;

    TaskJacobian = calculateOperationPointJacobian(currentOrigins.size()-1);
//           TaskJacobian.rightCols(3).setZero();
//            TaskJacobian.col(5).setZero();
//            TaskJacobian.col(3).setZero();

    double Kp = 1.;
    double Kv = 3;

    DesiredSpeedVector = Eigen::Vector3d((-Kp *
                    (endEffectorPos - myGoalPoint)).m_floats);


    // Conditioning the force
    if (DesiredSpeedVector.norm() > GLOBALBEHAVIOUR_MAXSPEED)
            DesiredSpeedVector *= (GLOBALBEHAVIOUR_MAXSPEED / DesiredSpeedVector.norm());
    Eigen::Vector3d ActualSpeedVector = TaskJacobian * myTaskLevelController->getSpeedVector();

    TaskReferenceAcceleration = -Kv * (ActualSpeedVector - DesiredSpeedVector);
//std::cout<<"myGoalPoint: "<<myGoalPoint[0]<<" "<<myGoalPoint[1]<<" "<<myGoalPoint[2]<<std::endl;
//    std::cout<<"DesiredSpeedVector= "<<DesiredSpeedVector.transpose()
//        <<"; ActualSpeedVector= "<<ActualSpeedVector.transpose()<<"; TaskReferenceAcceleration="<< TaskReferenceAcceleration.transpose()<<std::endl;

}








