/*
 * SecondaryBehavior.cpp
 *
 *  Created on: 30.05.2012
 *      Author: sergey
 */

#include "SecondaryBehavior.h"

SecondaryBehavior::SecondaryBehavior(Transformhandler* ReferenceTransformhandler,
                                     TaskLevelController* OwningTaskLevelController)
                                     : Behaviour(ReferenceTransformhandler,OwningTaskLevelController)
{
  BehaviorActive = true;
  OperationPointIndex = 5;

  OperationalDOFMask[MOVE_X] = true;
  OperationalDOFMask[MOVE_Y] = true;
  OperationalDOFMask[MOVE_Z] = true;
  OperationalDOFMask[ROTATE_X] = false;
  OperationalDOFMask[ROTATE_Y] = false;
  OperationalDOFMask[ROTATE_Z] = false;

}

SecondaryBehavior::~SecondaryBehavior()
{}

const torqueVector& SecondaryBehavior::getTorque()
{
        calculateReferenceAcceleration();
        calculateNextTaskNullspace();
    return myTorques;
}

void SecondaryBehavior::calculateReferenceAcceleration()
{
  Eigen::MatrixXd TaskJacobianBuf = calculateOperationPointJacobian(OperationPointIndex);
  TaskJacobian.setZero(TaskJacobianBuf.rows(),TaskJacobianBuf.cols());

  double Kp = 0.5;
  double Kv = 2;
  cVector q_current = BehaviorTransformhandler->getConfiguration();
  torqueVector Desired_q_speed;

  //keeping 2nd joint at zero
  Desired_q_speed[2] = - Kp * q_current[2];
  TaskJacobian.col(2) = TaskJacobianBuf.col(2);

  //keeping positive values for 3rd joint
  if (q_current[3] < 0)
  {
    Desired_q_speed[3] = - Kp * q_current[3];
    TaskJacobian.col(3) = TaskJacobianBuf.col(3);
  }

  torqueVector Desired_q_acceleration;
  Desired_q_acceleration.setZero();

  torqueVector Actual_q_speed = myTaskLevelController->getSpeedVector();
  Desired_q_acceleration = -Kv * (Actual_q_speed - Desired_q_speed);


  //TODO: Compute Coriolis and gravitation parts with Newton-Euler algorithm and
  //                    add their contribution to the torques
  torqueVector DesiredTorques = myTaskLevelController->getInertiaMatrix() * Desired_q_acceleration;
//std::cout<<"q_current: "<<q_current.transpose()<<std::endl;
// std::cout<<"DesiredTorques: "<<DesiredTorques.transpose()<<std::endl;
 Eigen::MatrixXd N_prev = myTaskLevelController->getNullSpaceMatrix();
  myTorques = N_prev.transpose() * DesiredTorques;
 // std::cout<<"after nullspace: "<<myTorques.transpose()<<std::endl;

}


