/*
 * ManipulabilityMaintainBehavior.cpp
 *
 *  Created on: 16.11.2011
 *      Author: sergey
 */

#include "StopBehavior.h"

StopBehavior::StopBehavior(Transformhandler* ReferenceTransformhandler,
		TaskLevelController* OwningTaskLevelController)
		: Behaviour(ReferenceTransformhandler,OwningTaskLevelController)
{
  BehaviorActive = true;
  OperationPointIndex = NumberOfDOF-1;
  CloseToValueZone = 0.0;
  desiredAngleValues_init << 0, M_PI/4, 0, M_PI/4, 0, -M_PI/4, 0;

  OperationalDOFMask[MOVE_X] = true;
  OperationalDOFMask[MOVE_Y] = true;
  OperationalDOFMask[MOVE_Z] = true;
  OperationalDOFMask[ROTATE_X] = false;
  OperationalDOFMask[ROTATE_Y] = false;
  OperationalDOFMask[ROTATE_Z] = false;
}

StopBehavior::~StopBehavior() {
}

/**
   @brief Calculates the joint torques and then returns them.

   NOTE: torques calculation has no notion of time limits
*/
const torqueVector& StopBehavior::getTorque()
{
	calculateReferenceAcceleration();
	calculateNextTaskNullspace();
    return myTorques;
}




/* ManipulabilityMaintainBehavior behavior works in configuration space and it should have the lowest priority
 * among behaviors, acting at the arm joints. It tries to keep the joint angles close to
 * to a desired value within range [-CloseToZeroZone, CloseToZeroZone];
 * All calculations are performed in one function; the Nullspace is modified explicitly by
 * disabling certain joints for next behaviors 	*/
void StopBehavior::calculateReferenceAcceleration()
{
  Eigen::MatrixXd TaskJacobianBuf = calculateOperationPointJacobian(OperationPointIndex);

  torqueVector Desired_q_acceleration;
  Desired_q_acceleration.setZero();

  double Kv = 6;
  torqueVector Actual_q_speed = myTaskLevelController->getSpeedVector();
  Desired_q_acceleration = -Kv * Actual_q_speed;


  //TODO: Compute Coriolis and gravitation parts with Newton-Euler algorithm and
  // 			add their contribution to the torques
  torqueVector DesiredTorques = myTaskLevelController->getInertiaMatrix() * Desired_q_acceleration;

  Eigen::MatrixXd N_prev = myTaskLevelController->getNullSpaceMatrix();
  myTorques = N_prev.transpose() * DesiredTorques;

  BehaviorActive = DesiredTorques.norm() > 0;
}




