/*
 * ConfToCellsMapper.cpp
 *
 *  Created on: 11.03.2012
 *      Author: sergey
 */

#include "ConfToCellsMapper.h"
#include "../SchunkTransformhandler.h"
#include <time.h>

ConfToCellsMapper::ConfToCellsMapper(map<long int, Cell> *cell_array)
{
  cells = cell_array;
  myTransformhandler = new SchunkTransformhandler();

  /*----------CREATION OF POINT ARRAY-----------*/

  double xmin,xmax,ymin,ymax,zmin,zmax;
  double x,y,z;
  double point_step = CELL_GRID_STEP/2;
  //**link 1**//
  xmin=-0.067;
  xmax=0.067;
  ymin = -0.067;
  ymax=0.067;
  zmin=0;
  zmax=0.301;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
    while (y<ymax){
      while (x<xmax){
        if (x*x+y*y < xmax*xmax)
          link_1_points.push_back(btVector3(x,y,z));
        x+=point_step;
      }
      y+=point_step;
      x=xmin;
    }
    z+=point_step;
    y=ymin;
  }

  //**link2**//
  xmin=-0.067;
  xmax=0.067;
  ymin = -0.067;
  ymax=0.067;
  zmin=-0.095;
  zmax=0.093;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
    while (y<=ymax){
      while (x<=xmax){
        if (x*x+y*y < xmax*xmax)
          link_2_points.push_back(btVector3(x,y,z));
        x+=point_step;
      }
      y+=point_step;
      x=xmin;
    }
    z+=point_step;
    y=ymin;
  }

  //**link3**//
  xmin=-0.057;
  xmax=0.057;
  ymin = -0.057;
  ymax=0.057;
  zmin=-0.11;
  zmax=0.219;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
   while (y<=ymax){
     while (x<=xmax){
       if (x*x+y*y < xmax*xmax)
         link_3_points.push_back(btVector3(x,y,z));
       x+=point_step;
     }
     y+=point_step;
     x=xmin;
   }
   z+=point_step;
   y=ymin;
  }

   //**link4**//
  xmin=-0.057;
  xmax=0.057;
  ymin = -0.057;
  ymax=0.057;
  zmin=-0.084;
  zmax=0.084;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
    while (y<=ymax){
      while (x<=xmax){
        if (x*x+y*y < xmax*xmax)
          link_4_points.push_back(btVector3(x,y,z));
        x+=point_step;
      }
      y+=point_step;
      x=xmin;
    }
    z+=point_step;
    y=ymin;
  }

  //**link5**//
  xmin=-0.045;
  xmax=0.045;
  ymin = -0.045;
  ymax=0.045;
  zmin=-0.100;
  zmax=0.177;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
   while (y<=ymax){
     while (x<=xmax){
       if (x*x+y*y < xmax*xmax)
         link_5_points.push_back(btVector3(x,y,z));
       x+=point_step;
     }
     y+=point_step;
     x=xmin;
   }
   z+=point_step;
   y=ymin;
  }

  //**link6**//
  xmin=-0.045;
  xmax=0.045;
  ymin = -0.045;
  ymax=0.045;
  zmin=-0.073;
  zmax=0.072;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
    while (y<=ymax){
      while (x<=xmax){
        if (x*x+y*y < xmax*xmax)
          link_6_points.push_back(btVector3(x,y,z));
        x+=point_step;
      }
      y+=point_step;
      x=xmin;
    }
    z+=point_step;
    y=ymin;
  }

  //**link7**//
  xmin=-0.038;
  xmax=0.038;
  ymin = -0.038;
  ymax=0.038;
  zmin=-0.172;
  zmax=0.000;
  x=xmin;y=ymin;z=zmin;
  while (z<=zmax){
   while (y<=ymax){
     while (x<=xmax){
       if (x*x+y*y < xmax*xmax)
         link_7_points.push_back(btVector3(x,y,z));
       x+=point_step;
     }
     y+=point_step;
     x=xmin;
   }
   z+=point_step;
   y=ymin;
  }

  cout<<"number of link1 points "<<link_1_points.size()<<endl;
  cout<<"number of link2 points "<<link_2_points.size()<<endl;
  cout<<"number of link3 points "<<link_3_points.size()<<endl;
  cout<<"number of link4 points "<<link_4_points.size()<<endl;
  cout<<"number of link5 points "<<link_5_points.size()<<endl;
  cout<<"number of link6 points "<<link_6_points.size()<<endl;
  cout<<"number of link7 points "<<link_7_points.size()<<endl;

  /*----------CREATION OF POINT ARRAY END-----------*/
}

set<long int> ConfToCellsMapper::mapConfigurationToCells(const Configuration &conf, double cell_size)
{
  myTransformhandler->setConfiguration(conf.getCvector());
  vector<btTransform> transforms = myTransformhandler->getRootToJointTransforms();
  set<long int> cell_IDs;

  vector<btVector3> transformed_points;
  for (uint j=0; j<link_1_points.size(); j++)
    transformed_points.push_back(transforms[0] * link_1_points[j]);
  for (uint j=0; j<link_2_points.size(); j++)
    transformed_points.push_back(transforms[1] * link_2_points[j]);
  for (uint j=0; j<link_3_points.size(); j++)
    transformed_points.push_back(transforms[2] * link_3_points[j]);
  for (uint j=0; j<link_4_points.size(); j++)
    transformed_points.push_back(transforms[3] * link_4_points[j]);
  for (uint j=0; j<link_5_points.size(); j++)
    transformed_points.push_back(transforms[4] * link_5_points[j]);
  for (uint j=0; j<link_6_points.size(); j++)
    transformed_points.push_back(transforms[5] * link_6_points[j]);
  for (uint j=0; j<link_7_points.size(); j++)
    transformed_points.push_back(transforms[6] * link_7_points[j]);

  for (uint i=0; i<transformed_points.size(); i++)
  {
    //determining cell center point
    int cell_x, cell_y, cell_z, fx,fy,fz;
    fx = floor(transformed_points[i][0]/cell_size);
    fy = floor(transformed_points[i][1]/cell_size);
    fz = floor(transformed_points[i][2]/cell_size);

    if (transformed_points[i][0] - fx*cell_size > cell_size/2)
      cell_x = (int)(cell_size*1000) * (fx+1);
    else
      cell_x = (int)(cell_size*1000) * fx;

    if (transformed_points[i][1] - fy*cell_size > cell_size/2)
      cell_y = (int)(cell_size*1000) * (fy+1);
    else
      cell_y = (int)(cell_size*1000) * fy;

    if (transformed_points[i][2] - fz*cell_size > cell_size/2)
      cell_z = (int)(cell_size*1000) * (fz+1);
    else
      cell_z = (int)(cell_size*1000) * fz;

    //calculating cell ID
    long int cellID = abs(cell_z) + abs(cell_y)*1e5 + abs(cell_x)*1e9;
    if (cell_z < 0)
      cellID += 1e4;
    if (cell_y < 0)
      cellID += 1e8;
    if (cell_x < 0)
      cellID += 1e12;

    if (!cell_IDs.count(cellID))
      cell_IDs.insert(cellID);
  }

  return cell_IDs;
}

void ConfToCellsMapper::mapNodeToCells(const Node &n, double cell_size)
{
  set<long int> a;
  set<long int>::iterator it;
  for(uint i=0; i<Node::NUMBER_OF_CONFIGS; i++)
  {
    if(n.config[i].exist)
    {
      a = this->mapConfigurationToCells(n.config[i],cell_size);
      for(it = a.begin(); it!=a.end(); it++)
      {
        if (cells->count(*it)!=0)
        {
          cells->find(*it)->second.addConfig(pair<long int, uint>(n.ID,i));
         // cout<<"Added config "<<n.ID<<"."<<i<<" to cell "<<*it<<endl;
        }
        else
          cout<<"cell "<<(*it)<<" not found!!!"<<endl;
      }
    }
  }
}

ConfToCellsMapper::~ConfToCellsMapper()
{
  delete myTransformhandler;
}
