/*
 * IKSolver.cpp
 *
 *  Created on: 05.03.2012
 *      Author: sergey
 */

#include "IKSolver.h"
#include <math.h>
#include <sstream>
#include "../SchunkTransformhandler.h"


IKSolver::IKSolver(urdf::Model my_model)
{
   JointLimits.resize(6,2);

   transforms = new SchunkTransformhandler();
  for (int i=0;i<6;i++)
  {
    std::stringstream ss;
    ss << "schunk_"<<i+1<<"_joint";

    JointLimits(i,0) = my_model.getJoint(ss.str())->limits->lower;
    JointLimits(i,1) = my_model.getJoint(ss.str())->limits->upper;
  }
  //TODO: get data from model
  l1=0.3;
  l2=0.328;
  l3=0.2765;
  l4=0.172;

}

IKSolver::~IKSolver()
{
}

bool IKSolver::solveIKtask(btVector3 point, btVector3 orient, Configuration& conf)
{
  if ((orient.length()!=1) || (point.length() == 0))
    return false;
  cVector result;
  result.setZero();

  btVector3 joint6point = point - l4*orient;

  result(0) = -atan2(joint6point.y(),joint6point.x());
  result(2) = 0;
  double dist2 = pow(joint6point.x(),2) + pow(joint6point.y(),2) + pow(joint6point.z()-l1, 2);
  if (dist2 > pow(l2+l3,2))
    return false;
  result(3) = M_PI - acos((pow(l2,2)+pow(l3,2)-dist2)/(2*l2*l3));
  result(1) = M_PI_2 - acos((dist2 + pow(l2,2)-pow(l3,2))/(2*l2*sqrt(dist2))) - asin((joint6point.z()-l1)/sqrt(dist2));

  transforms->setConfiguration(result);

  /****ALGEBRAIC*********/

  btVector3 VectorFromFrame4 = transforms->getRootToJointTransform(6).getBasis().inverse()*orient;
  btMatrix3x3 mat = transforms->getRootToJointTransform(6).getBasis();
  double x=VectorFromFrame4[0],y=VectorFromFrame4[1],z=VectorFromFrame4[2];
  VectorFromFrame4 = transforms->getRootToJointTransform(4).getBasis()*btVector3(0,0,1);

  double theta, phi;
  phi = acos(z);

  theta = -atan2(y,x);
  result(4) = theta;
  result(5) = phi;

  for (int i=0; i<6;i++)
    if (result(i)<JointLimits(i,0) || result(i)>JointLimits(i,1)
        || isnan(result(i)) || isnan(-result(i)))
    {
      result.setZero();
      conf.setCvector(result);
      conf.exist = false;
      return false;
    }

  conf.setCvector(result);
  conf.exist = true;
//  conf.valid = true;
  return true;
}

void IKSolver::computeIKForNode(Node &n)
{
  n.config[Node::X_NEG].exist = this->solveIKtask(n.position,btVector3(-1,0,0),n.config[Node::X_NEG]);
  n.config[Node::X_POS].exist = this->solveIKtask(n.position,btVector3(1,0,0),n.config[Node::X_POS]);
  n.config[Node::Y_NEG].exist = this->solveIKtask(n.position,btVector3(0,-1,0),n.config[Node::Y_NEG]);
  n.config[Node::Y_POS].exist = this->solveIKtask(n.position,btVector3(0,1,0),n.config[Node::Y_POS]);
  n.config[Node::Z_NEG].exist = this->solveIKtask(n.position,btVector3(0,0,-1),n.config[Node::Z_NEG]);
  n.config[Node::Z_POS].exist = this->solveIKtask(n.position,btVector3(0,0,1),n.config[Node::Z_POS]);
}




