#ifndef __TRANSFORMHANDLER_H__
#define __TRANSFORMHANDLER_H__


#include <Eigen/Core>
#include <pthread.h>
#include <vector>
#include "matrixDefinition.h"
#include <btBulletDynamicsCommon.h>
#include "Spatial.h"

/**
   @brief Superclass that defines an interface for common accessing and updating
   of kinematic transforms.
*/
class Transformhandler {
 public:
  Transformhandler();
  /** @brief Updates myTransforms according to myConfiguration */
  virtual void updateTransforms()=0;
  std::vector<btTransform> getTransforms();
  std::vector<btTransform> getRootToJointTransforms();
  btTransform getRootToJointTransform(uint i);
  virtual std::vector<btVector3> getAxes();
  virtual std::vector<btMatrix3x3> getBasises();
  virtual std::vector<btVector3> getOrigins();
  const Eigen::MatrixXd getJacobian();
  void setConfiguration(const cVector& config);
  cVector getConfiguration();
  std::vector<SpatialInertiaMatrix6d>* getJointInertia();
  void getJointLimits(Eigen::MatrixXd& limits, Eigen::VectorXi& isLimitedArray );

  EIGEN_MAKE_ALIGNED_OPERATOR_NEW //Aligning bytes in matrices (libEigen tricks)
 protected:
  virtual void CalculateJacobian() = 0;
  virtual void InitJointInertia() = 0;
  virtual void InitJointLimits() = 0;
  /** @brief Chain of transforms */
  std::vector<btTransform> myTransforms;

  std::vector<btTransform> rootToJointTransforms;
  /** @brief Convenience method to handle concurrent access */
  void setTransforms(const std::vector<btTransform>& transforms);
  cVector myConfiguration;
  bool shouldUpdate;
  pthread_mutex_t transformMutex;
  pthread_mutex_t configMutex;
  pthread_mutex_t JacobianMutex;
  Eigen::MatrixXd myJacobian;

  std::vector<SpatialInertiaMatrix6d> JointInertia;
  Eigen::MatrixXd myJointLimits;
  Eigen::VectorXi jointIsLimited;
};

#endif /*__TRANSFORMHANDLER_H__*/
