/*
 * main.cpp
 *
 *  Created on: 16.04.2012
 *      Author: sergey
 */

#include <ros/ros.h>
#include <ros/console.h>
#include "matrixDefinition.h"
#include "Planner.h"
#include "Behaviours/AvoidanceBehaviour.h"
#include <visualization_msgs/MarkerArray.h>

int main(int argc, char** argv)
{

  std::cout<<std::setiosflags(std::ios::fixed) << std::setprecision(3);
  cout<<"Planner Started"<<endl;

  ros::init(argc, argv, "online_planner");
  ros::NodeHandle n;

  ros::Rate delay(4);

  Planner* planner = new Planner(&n);
  cVector initConfig;
 // initConfig<< 0, 1.5*M_PI/4, 0, 1.5*M_PI/3, 0, -M_PI/4, 0;
  initConfig<< 0, 1.5*M_PI/4, 0, 1.5*M_PI/3, M_PI, M_PI/4, 0;
  planner->getRobotController()->ResetConfiguration(initConfig);

  ObstaclesHandler* obstacles = planner->getObstaclesHandler();

//exp 8
  //btVector3 obst1_pos = btVector3(0.5,0.15,0.3);

 // btVector3 obst1_pos = btVector3(0.4,0.1,0.05);
  btVector3 obst1_pos = btVector3(0.4,0.2,0.2);
 // Obstacle* obst1 = obstacles->addObstacle(Obstacle::SPHERE, btVector3(0.1,0.1,0.1),obst1_pos);
  Obstacle* obst1 = obstacles->addObstacle(Obstacle::CUBE, btVector3(0.5,0.1,0.1),obst1_pos);

  //exper.8
 // Obstacle* obst1 = obstacles->addObstacle(Obstacle::CUBE, btVector3(0.2,0.1,0.02),obst1_pos);
//exper 6
 // Obstacle* obst1 = obstacles->addObstacle(Obstacle::SPHERE, btVector3(0.2,0.4,0.1),obst1_pos);

  btVector3 EEpointingGoal = btVector3(1,0.2,0.2);

  btVector3 GlobalGoal = btVector3(0.4,0.4,0.2);
  //exp 8
  //btVector3 GlobalGoal = btVector3(0.6,0.15,0.25);

  planner->setNewEEaimingGoal(EEpointingGoal);

  //planner->setEEOrientationGoal(btVector3(1,0,0));
  //planner->setNewGlobalGoal(btVector3(0.4,0.4,0.2));
//  ros::Rate delay1s(1);
//  delay1s.sleep();
//  delay1s.sleep();
//  delay1s.sleep();

  planner->setNewGlobalGoal(GlobalGoal);

//  planner->setNewGlobalGoal(btVector3(0.4,0.4,0.2));

  //ros::Rate loop_rate(2);
  ros::Rate loop_rate(2);
  bool exitFlag = false;
  int ctr=0;

  ros::Publisher marker_pub = n.advertise<visualization_msgs::MarkerArray>("visualization_marker_array", 10);


  while (true) //ros::ok())
  {
    ros::spinOnce();
    exitFlag = planner->checkExecutionStatus();
//    if (exitFlag)
//      break;
    if (ctr % 4 == 0){
     obstacles->moveObstacle(obst1,obst1_pos);
  //   if (obst1_pos[2]<0.2)
  //    obst1_pos[1]+= -0.005;
    }
    ctr++;

    //**************MARKER*******///

      visualization_msgs::MarkerArray marker;
      marker.markers.resize(2);

      marker.markers[0].header.stamp = ros::Time::now();
      marker.markers[0].header.frame_id = "/fix";
      marker.markers[0].ns = "EEpointingGoal";
      marker.markers[0].id = 0;
      marker.markers[0].type =visualization_msgs::Marker::SPHERE;
      marker.markers[0].action = visualization_msgs::Marker::ADD;
      marker.markers[0].pose.position.x = EEpointingGoal.x();
      marker.markers[0].pose.position.y = EEpointingGoal.y();
      marker.markers[0].pose.position.z = EEpointingGoal.z();

      marker.markers[0].lifetime = ros::Duration();
      marker.markers[0].scale.x = 0.05;
      marker.markers[0].scale.y = 0.05;
      marker.markers[0].scale.z = 0.05;
      marker.markers[0].color.r = 1.0f;
      marker.markers[0].color.g = 1.0f;
      marker.markers[0].color.b = 0.0f;
      marker.markers[0].color.a = 1.0;

      btQuaternion a(btVector3(0,0,1),0);
      marker.markers[0].pose.orientation.w = a.getW();
      marker.markers[0].pose.orientation.x = a.getX();
      marker.markers[0].pose.orientation.y = a.getY();
      marker.markers[0].pose.orientation.z = a.getZ();

      marker.markers[1].header.stamp = ros::Time::now();
      marker.markers[1].header.frame_id = "/fix";
      marker.markers[1].ns = "GlobalGoal";
      marker.markers[1].id = 0;
      marker.markers[1].type =visualization_msgs::Marker::SPHERE;
      marker.markers[1].action = visualization_msgs::Marker::ADD;
      marker.markers[1].pose.position.x = GlobalGoal.x();
      marker.markers[1].pose.position.y = GlobalGoal.y();
      marker.markers[1].pose.position.z = GlobalGoal.z();

      marker.markers[1].lifetime = ros::Duration();
      marker.markers[1].scale.x = 0.03;
      marker.markers[1].scale.y = 0.03;
      marker.markers[1].scale.z = 0.03;
      marker.markers[1].color.r = 1.0f;
      marker.markers[1].color.g = 0.0f;
      marker.markers[1].color.b = 0.0f;
      marker.markers[1].color.a = 1.0;

      marker.markers[1].pose.orientation.w = a.getW();
      marker.markers[1].pose.orientation.x = a.getX();
      marker.markers[1].pose.orientation.y = a.getY();
      marker.markers[1].pose.orientation.z = a.getZ();

      if (ros::ok())
        marker_pub.publish(marker);

      //************END MARKER********///////


    loop_rate.sleep();
  }

  cout<<"while_main out"<<endl;

  planner->getRobotController()->setObstacleHandler(NULL);
  delete obstacles;
  delay.sleep();
  n.shutdown();
  delay.sleep();
  delete planner;

  cout<<"Online planner execution finished normally"<<endl;
  return 0;
}






